package cz.muni.fi.pa165.core.common;

import jakarta.persistence.EntityNotFoundException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

public abstract class DomainService<T extends DomainObject> {

  public static final int DEFAULT_PAGE_SIZE = 10;

  public abstract JpaRepository<T, String> getRepository();

  @Transactional
  public T create(T entity)
  {
    entity.createdDateTime = LocalDateTime.now();
    return getRepository().save(entity);
  }

  @Transactional(readOnly = true)
  public Page<T> findAllPageable(Pageable pageable) {
    return getRepository().findAll(pageable);
  }

  @Transactional(readOnly = true)
  public List<T> findAll() {
    return getRepository().findAll().stream().filter(entity -> entity.deletedDateTime == null).toList();
  }

  @Transactional(readOnly = true)
  public Page<T> findAllPageableInt(int page) {
    return getRepository().findAll(PageRequest.of(page, DEFAULT_PAGE_SIZE))/*.filter(entity -> entity.deletedDateTime == null)*/;
  }

  @Transactional(readOnly = true)
  public T findById(String id) {
    return getRepository()
            .findById(id)
            .filter(entity -> entity.deletedDateTime == null)
        .orElseThrow(() -> new EntityNotFoundException("Entity with '" + id + "' not found."));
  }

  @Transactional
  public void deleteAll()
  {
    List<T> entities = findAll();
    entities.stream().map(entity -> entity.deletedDateTime = LocalDateTime.now());
    getRepository().saveAll(entities);
  }

  @Transactional
  public T deleteById(String id) {
    T entity = findById(id);
    if (entity == null)
      throw new EntityNotFoundException("Entity '" + id + "' not found.");
    entity.deletedDateTime = LocalDateTime.now();
    getRepository().save(entity);
    return entity;
  }

  @Transactional
  public T update(T entityToUpdate, String id) {
    T entity = findById(id);
    if (entity == null)
      throw new EntityNotFoundException("Entity '" + id + "' not found.");
    // TODO: change when ORM tool available
    entityToUpdate.setId(id);
    getRepository().save(entityToUpdate);
    return entity;
  }

  @Transactional
  public void deleteAllHardDelete()
  {
    getRepository().deleteAll();
  }
}
