package cz.muni.fi.pa165.core.user.roles;

import cz.muni.fi.pa165.model.dto.common.Result;
import cz.muni.fi.pa165.model.dto.role.RoleDto;
import cz.muni.fi.pa165.model.dto.user.UserCreateDto;
import cz.muni.fi.pa165.model.dto.user.UserDto;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.swing.*;
import java.util.List;

@RestController
@RequestMapping("/api/role")
public class RoleController {

    private final RoleService roleService;
    @Autowired
    public RoleController(RoleService roleService){
        this.roleService = roleService;
    }
    @Operation(
            summary = "Create role for user",
            description = "Creates a new user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "201",
                            description = "Role created",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = UserDto.class))
                            }),
                    @ApiResponse(responseCode = "400", description = "Invalid input", content = @Content),
            })
    @PostMapping
    public Role create(
            @Parameter(description = "Role to be created") @RequestBody Role roleDto) {
        return roleService.create(roleDto);
    }

    @DeleteMapping("/{id}")
    @Operation(summary = "Delete a user role")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Role deleted successfully"),
            @ApiResponse(responseCode = "404", description = "Role not found"),
            @ApiResponse(responseCode = "500", description = "Internal server error")
    })
    public Role deleteById(
            @Parameter(description = "ID of the role to be deleted") @PathVariable String id) {
        return roleService.deleteById(id);
    }

    @Operation(
            summary = "Get all user roles",
            description = "Returns all user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "200",
                            description = "Roles found",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = Result.class))
                            })
            })
    @GetMapping
    public List<Role> findUserRoles(
            @Parameter(description = "Page number of results to retrieve") @RequestParam String userId) {
        return roleService.findAllByUser(userId);
    }

    @Operation(
            summary = "Get all user roles",
            description = "Returns all user roles",
            tags = {"role"})
    @ApiResponses(
            value = {
                    @ApiResponse(
                            responseCode = "200",
                            description = "Roles found",
                            content = {
                                    @Content(
                                            mediaType = "application/json",
                                            schema = @Schema(implementation = Result.class))
                            })
            })
    @GetMapping("/all")
    public List<Role> findAll(
            @Parameter(description = "Page number of results to retrieve") @RequestParam String userId) {
        return roleService.findAll();
    }
}
