package cz.muni.fi.pa165.statistics.statistics;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpHead;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.impl.client.HttpClientBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.ProtocolException;
import java.net.URL;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

import org.apache.http.entity.StringEntity;
import org.springframework.transaction.annotation.Transactional;

@Service
public class StatisticService {

    private final StatisticRepository repository;

    @Autowired
    public StatisticService(StatisticRepository repository){
        this.repository = repository;
    }

    @Transactional
    public UserStatistic CreateStatistic(UserStatistic userStatistic) throws IOException {
        UserStatistic statistic = repository.findUserHouseDateStatistics(userStatistic.getUserId(), userStatistic.getHouseId(), userStatistic.getMonthNum(), userStatistic.getYearNum());

        if (statistic == null){
            userStatistic.setId(UUID.randomUUID().toString());
            userStatistic.setMonthlyConsumption(GenerateStatistic(userStatistic));
            repository.save(userStatistic);
            return userStatistic;
        }

        return statistic;
    }

    @Transactional
    public List<UserStatistic> findAllUserStatistics(String userId)
    {
        return repository.findAllByUser(userId);
    }

    @Transactional
    public List<UserStatistic> findAllUserStatisticsByHouse(String userId, String houseId)
    {
        return repository.findAllByUserAndHouse(userId, houseId);
    }

    private double GenerateStatistic(UserStatistic userStatistic) throws IOException {
        // TODO: get first and last day of month
        LocalDateTime startDateTime = LocalDateTime.of(2023,3,1,0,0,0,1);
        LocalDateTime lastDateTime = LocalDateTime.of(2023,3,30,0,0,0,1);

        String payload = String.format("""
                {
                    "id": "%s",
                    "houseId": "%s",
                    "startTime": "%s"
                    "endTime": "%s"
                }
                """, userStatistic.getId(),
                userStatistic.getHouseId(),
                startDateTime,
                lastDateTime);
        StringEntity entity = new StringEntity(payload,
                ContentType.APPLICATION_FORM_URLENCODED);

        HttpClient httpClient = HttpClientBuilder.create().build();
        HttpPost request = new HttpPost("http://localhost:8080/api/user/statistics");
        request.setEntity(entity);
        request.setHeader("Content-Type", "application/json");
        request.setHeader("accept:", "*/*");

        HttpResponse response = httpClient.execute(request);
        System.out.println(response.getStatusLine().getStatusCode());
        HttpEntity content = response.getEntity();
        //System.out.println(content.);
        return 0;
    }
}
