package cz.muni.fi.pa165.statistics.statistics;

import io.swagger.v3.oas.annotations.Operation;
import jakarta.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;

@RestController
@RequestMapping("/api/user-statistics")
public class UserStatisticsController {

    private final StatisticService statisticService;
    private final UserStatisticsMapper mapper;

    @Autowired
    public UserStatisticsController(StatisticService statisticService, UserStatisticsMapper mapper)
    {
        this.statisticService = statisticService;
        this.mapper = mapper;
    }

    @Operation(
            summary = "Create statistics",
            description = "Creates a new statistics.")
    @PostMapping
    public ResponseEntity create(@RequestBody @Valid StatisticCreateDto createDto) throws IOException {
        /*try
        {*/
            statisticService.CreateStatistic(mapper.fromDto(createDto));
            return ResponseEntity.ok().build();
        /*}
        catch (Exception e)
        {*/
           // return ResponseEntity.internalServerError(e.getMessage()).build();
        //}
    }

    @Operation(
            summary = "Get user statistics",
            description = "Get user statistics.")
    @GetMapping("/all")
    public List<UserStatistic> findAllByUser(@RequestParam @Valid String userId) {
        return statisticService.findAllUserStatistics(userId);
    }

    @Operation(
            summary = "Get user statistics for house",
            description = "Get user statistics for house.")
    @GetMapping("/user")
    public List<UserStatistic> findHouseStatisticsByUser(@RequestParam @Valid String userId,
                                                         @RequestParam @Valid String houseId) {
        return statisticService.findAllUserStatistics(userId);
    }
}
